@@ -11,6 +11,7 @@
 {-# LANGUAGE NoMonomorphismRestriction  #-}
 {-# LANGUAGE GADTs                      #-}
 {-# LANGUAGE TypeFamilies               #-}
+{-# LANGUAGE MultiParamTypeClasses      #-}
 {-# LANGUAGE ScopedTypeVariables        #-}
 {-# LANGUAGE RecordWildCards            #-}
 {-# LANGUAGE TupleSections              #-}
@@ -51,11 +52,13 @@ import Control.Lens hiding ((...), (<.>))
 
 import qualified Data.Vector as Arr
 import Data.List (sort)
+import Data.List.NonEmpty (NonEmpty(..))
 
 import Data.VectorSpace
 import Data.Basis
+import Math.LinearMap.Category
 import Data.AffineSpace
-import Data.LinearMap.HerMetric
+import Data.VectorSpace.Free ()
 import Data.Manifold.PseudoAffine
 import Data.Manifold.TreeCover
 import Data.Semigroup
@@ -66,60 +69,6 @@ import Control.DeepSeq
 type R2 = Dia.V2 Double
 type P2 = Dia.P2 Double
 
-instance AffineSpace R2 where
-  type Diff R2 = R2
-  (.-.) = (Dia.^-^)
-  (.+^) = (Dia.^+^)
-instance AdditiveGroup R2 where
-  (^+^) = (Dia.^+^)
-  zeroV = Dia.zero
-  negateV = Dia.negated
-instance VectorSpace R2 where
-  type Scalar R2 = Double
-  (*^) = (Dia.*^)
-instance HasBasis R2 where
-  type Basis R2 = Either () ()
-  basisValue (Left ()) = 1^&0
-  basisValue (Right ()) = 0^&1
-  decompose v = [(Left(), v^._x), (Right(), v^._y)]
-  decompose' v (Left ()) = v^._x
-  decompose' v (Right ()) = v^._y
-instance InnerSpace R2 where
-  (<.>) = Dia.dot
-instance FiniteDimensional R2 where
-  dimension = Tagged 2
-  basisIndex = Tagged bi where bi b = if (basisValue b::R2)^._x > 0.5 then 0 else 1
-  indexBasis = Tagged ib
-   where ib 0 = bx; ib 1 = by
-         [(bx,_), (by,_)] = decompose (1^&1 :: R2)
-  completeBasis = Tagged . fmap fst $ decompose (1^&1 :: R2)
-instance HasMetric' R2 where
-  type DualSpace R2 = R2
-  (<.>^) = (<.>)
-  functional f = f(1^&0) ^& f(0^&1)
-  doubleDual = id; doubleDual' = id
-instance Semimanifold R2 where
-  type Needle R2 = R2
-  fromInterior = id
-  toInterior = pure
-  translateP = Tagged (^+^)
-  (.+~^) = (^+^)
-instance PseudoAffine R2 where
-  p.-~.q = pure(p^-^q)
-
-instance AffineSpace P2 where
-  type Diff P2 = R2
-  (.-.) = (Dia..-.)
-  (.+^) = (Dia..+^)
-instance Semimanifold P2 where
-  type Needle P2 = R2
-  fromInterior = id
-  toInterior = pure
-  translateP = Tagged (.+^)
-  (.+~^) = (.+^)
-instance PseudoAffine P2 where
-  p.-~.q = pure(p.-.q)
-
 
 
 (^) :: Num n => n -> Int -> n
@@ -195,7 +144,7 @@ linFitMeanInCtrdUnitIntv (LinFitParams{..}) = constCoeff
 
 
 
-data DevBoxes y = DevBoxes { deviations :: HerMetric' (Diff y)
+data DevBoxes y = DevBoxes { deviations :: Metric' y
                            , maxDeviation :: Scalar (Diff y)   }
                 
 
@@ -229,7 +178,7 @@ type (x-.^>y) = RecursivePCM () x y
 
 recursiveSamples' :: forall x y v t .
           ( VectorSpace x, Real (Scalar x)
-          , AffineSpace y, v~Diff y, InnerSpace v, HasMetric v, RealFloat (Scalar v) )
+          , AffineManifold y, v~Needle y, HilbertSpace v, RealFloat (Scalar v) )
                      => PCMRange x -> [(y,t)] -> RecursiveSamples x y t
 recursiveSamples' xrng_g ys = calcDeviations . go xrng_g $ presplitList ys
     where go :: PCMRange x -> SplitList (y,t) -> RecursiveSamples' (Arr.Vector y) x y t
@@ -261,8 +210,8 @@ recursiveSamples' xrng_g ys = calcDeviations . go xrng_g $ presplitList ys
            where cdvs lPFits rPFits
                          rPCM@( RecursivePCM pFit dtls _ sSpc@(PCMRange xl wsp) slLn pts )
                     = RecursivePCM pFit dtls' (DevBoxes stdDev maxDev) sSpc slLn ()
-                   where stdDev = (^/ fromIntegral slLn) . sumV $ projector' <$> msqs
-                         maxDev =     sqrt           . maximum $ magnitudeSq <$> msqs
+                   where stdDev = scaleNorm (1/ fromIntegral slLn) $ spanNorm msqs
+                         maxDev =     sqrt        . maximum $ magnitudeSq <$> msqs
                          msqs = [ (y .-. ff x)
                                 | (x,y) <- normlsdIdd $ SplitList pts ]
                          ff = l₀splineRep (Pair lPFits rPFits) rPCM
@@ -281,12 +230,12 @@ rRoute (RecursivePCM {details = Left (Pair _ r)}) = Just r
                          
 
 recursiveSamples :: 
-          ( AffineSpace y, v~Diff y, InnerSpace v, HasMetric v, RealFloat (Scalar v) )
+          ( AffineManifold y, v~Needle y, HilbertSpace v, RealFloat (Scalar v) )
                      => [(y,t)] -> RecursiveSamples Int y t
 recursiveSamples = recursiveSamples' (PCMRange 0 1)
 
 recursivePCM :: ( VectorSpace x, Real (Scalar x)
-                , AffineSpace y, v~Diff y, InnerSpace v, HasMetric v, RealFloat (Scalar v) )
+                , AffineManifold y, v~Needle y, HilbertSpace v, RealFloat (Scalar v) )
                      => PCMRange x -> [y] -> x-.^>y
 recursivePCM xrng_g = recursiveSamples' xrng_g . fmap (,())
 
@@ -335,7 +284,7 @@ l₀splineRep (Pair lPFits rPFits)
 
 
 
-rPCMSample :: (AffineSpace y, v~Diff y, InnerSpace v, HasMetric v, RealFloat (Scalar v))
+rPCMSample :: (AffineManifold y, v~Needle y, HilbertSpace v, RealFloat (Scalar v))
        => Interval R -> R -> (R->y) -> R-.^>y
 rPCMSample (Interval l r) δx f = recursivePCM (PCMRange l δx) [f x | x<-[l, l+δx .. r]] 
                    
@@ -348,7 +297,7 @@ rPCM_R2_boundingBox rPCM@(RecursivePCM pFit _ (DevBoxes dev _) _ _ ())
            -*| Interval (yb - uy*2) (yt + uy*2)
  where pm = constCoeff pFit
        p₀ = pm .-^ linCoeff pFit; pe = pm .+^ linCoeff pFit
-       ux = metric' dev $ 1^&0; uy = metric' dev $ 0^&1
+       ux = dev |$| 1^&0; uy = dev |$| 0^&1
        [xl,xr] = sort[p₀^._x, pe^._x]; [yb,yt] = sort[p₀^._y, pe^._y]
 
 
@@ -416,6 +365,25 @@ instance Show GraphWindowSpecR2 where
                                \xResolution="++show xResolution++", \
                                \yResolution="++show yResolution++"}"
 
+windowCenter :: Lens' GraphWindowSpecR2 (R,R)
+windowCenter = lens
+    (\(GraphWindowSpecR2 l r b t _ _ _) -> ((l+r)/2, (b+t)/2))
+    (\(GraphWindowSpecR2 l r b t xRes yRes colSch) (cx, cy)
+         -> let rx = (r-l)/2; ry = (t-b)/2
+            in GraphWindowSpecR2 (cx - rx) (cx + rx) (cy - ry) (cy + ry)
+                                 xRes yRes colSch
+    )
+windowDiameter :: Lens' GraphWindowSpecR2 R
+windowDiameter = lens
+    (\(GraphWindowSpecR2 l r b t _ _ _) -> sqrt $ (r-l)^2 + (t-b)^2)
+    (\(GraphWindowSpecR2 l r b t xRes yRes colSch) dNew
+         -> let cx = (l+r)/2; rx = (r-l)/2
+                cy = (b+t)/2; ry = (t-b)/2
+                dOld = 2 * sqrt (rx^2 + ry^2)
+                η = dNew / dOld
+            in GraphWindowSpecR2 (cx - η*rx) (cx + η*rx) (cy - η*ry) (cy + η*ry)
+                                 xRes yRes colSch
+    )
 
 
 
@@ -470,8 +438,8 @@ xyRanges bb = let Just (c₁, c₂) = DiaBB.getCorners bb
 
 shadeExtends :: Shade P2 -> (Interval R, Interval R)
 shadeExtends shade
-      = ( (ctr^._x) ± sqrt (metric' expa $ 1^&0)
-        , (ctr^._y) ± sqrt (metric' expa $ 0^&1) )
+      = ( (ctr^._x) ± (expa |$| 1^&0)
+        , (ctr^._y) ± (expa |$| 0^&1) )
  where ctr = shade^.shadeCtr; expa = shade^.shadeExpanse
 
 
@@ -504,3 +472,8 @@ fromInt = fromIntegral
 
 
 
+
+newtype Latest a = Latest { getLatestOf :: NonEmpty a }
+ deriving (Hask.Functor)
+
+
